from aio_client.provider.api import PostProviderRequest
from aio_client.provider.api import push_request

from concentrator.smev3_v321.base.constants import SUCCESS_MESSAGE
from concentrator.smev3_v321.base.utils import BaseExecutor
from concentrator.smev3_v321.base.utils import SMEV3Response
from concentrator.smev3_v321.base.utils import is_cancel_allowed
from concentrator.smev3_v321.base.utils import render_type2xml
from concentrator.smev3_v321.constants import ERROR_ORG_CODE
from concentrator.smev3_v321.models import ApplicantAnswer
from concentrator.smev3_v321.service_types import kinder_conc
from kinder.core.declaration.models import Declaration
from kinder.core.direct.models import DRS
from kinder.core.direct.models import Direct

from .constants import ApplicationRejectionMessage
from ..helpers import get_declaration_by_client_or_portal_id


class ApplicationRejectionRequestExecutor(BaseExecutor):
    """Исполнитель сервиса ApplicationRejectionRequest."""

    name_service = 'ApplicationRejectionRequest'
    type_service = kinder_conc.ApplicationRejectionRequestType

    @classmethod
    def process(cls, message, request_body, **kwargs):

        response_body = render_type2xml(
            cls.get_response(request_body.ApplicationRejectionRequest),
            name_type='FormDataResponse')

        response = push_request(PostProviderRequest(
            origin_message_id=message['origin_message_id'],
            body=response_body,
            message_type=message['message_type'],
            replay_to=message['replay_to'],
            is_test_message=message['is_test_message']))

        return SMEV3Response(
            response,
            {
                'method_name': cls.name_service,
                'response': response_body,
                'result': SUCCESS_MESSAGE
            })

    @classmethod
    def get_response(cls, application_request):
        """Формирует и возвращает тело ответа."""

        order_id = application_request.orderId
        declaration_query = Declaration.objects.select_related(
            'status').prefetch_related('direct_set')

        declaration, _ = get_declaration_by_client_or_portal_id(
            declaration_query, order_id)

        if not declaration:
            return cls.get_bad_response(
                order_id, ApplicationRejectionMessage.DECL_NOT_FOUND)

        try:
            direct = declaration.direct_set.get(
                status__code__in=[DRS.REGISTER, DRS.DOGOVOR]
            )
        except Direct.MultipleObjectsReturned:
            return cls.get_bad_response(
                order_id, ApplicationRejectionMessage.MULTIPLY_DIRECT_FOUND)
        except Direct.DoesNotExist:
            return cls.get_bad_response(
                order_id, ApplicationRejectionMessage.DIRECT_NOT_FOUND)

        ApplicantAnswer.objects.update_or_create(
            direct=direct,
            defaults=dict(answer=False, comment=application_request.comment),
        )

        response = kinder_conc.FormDataResponseType(
            changeOrderInfo=kinder_conc.changeOrderInfoType(
                orderId=kinder_conc.orderIdType(order_id),
                statusCode=kinder_conc.statusCodeType(
                    ApplicationRejectionMessage.ORG_CODE_140),
                comment=ApplicationRejectionMessage.REJECT_ACCEPTED,
                cancelAllowed=is_cancel_allowed(declaration.status)
            )
        )

        return response

    @staticmethod
    def get_bad_response(order_id, comment):
        """
        Формирует тело ответа в случае ошибки.

        :param order_id: Идентификатор заявки из запроса
        :type order_id: int
        :param comment: Комментарий из запроса
        :type comment: str

        """

        response = kinder_conc.FormDataResponseType(
            changeOrderInfo=kinder_conc.changeOrderInfoType(
                orderId=kinder_conc.orderIdType(order_id),
                statusCode=kinder_conc.statusCodeType(ERROR_ORG_CODE),
                comment=comment
            )
        )

        return response
